#!/bin/bash

GXP_PHONE_IP="$1"

if [ -n "$2" ]; then
	GXP_PASSWD="$2"
else
	GXP_PASSWD="admin" # default
fi

get_sid() {
        while read -r || [[ -n "$REPLY" ]]
        do
                if [ -n "$sid" ]; then
                        break
                fi
                if [ "${REPLY:0:23}" == "Set-Cookie: session_id="  ]; then
                        sid="${REPLY:23}"
			sid="${sid//;}"
                fi
                if [ "${REPLY//'"sid"'}" != "$REPLY" ]; then
                        sid="${REPLY#*'"sid"'}"
			sid="${sid#*'"'}"
                        sid="${sid%%'"'*}"
                fi
        done
        echo "$sid"
}

post_status() {
        # Older firmware says need to reboot, newer firmware says success
        while read -r || [[ -n "$REPLY" ]]
        do
                if [ "${REPLY//eboot}" != "$REPLY" ]; then
                        echo "1"
                        break
                fi
                if [ "${REPLY//success}" != "$REPLY" ]; then
                        echo "1"
                        break
                fi
        done
}

urlencode () {
        tab="`echo -en "\x9"`"
        i="$@"
        i=${i//%/%25}  ; i=${i//' '/%20} ; i=${i//$tab/%09}
        i=${i//!/%21}  ; i=${i//'"'/%22}  ; i=${i//#/%23}
        i=${i//\$/%24} ; i=${i//\&/%26}  ; i=${i//\'/%27}
        i=${i//(/%28}  ; i=${i//)/%29}   ; i=${i//\*/%2a}
        i=${i//+/%2b}  ; i=${i//,/%2c}   ; i=${i//-/%2d}
        i=${i//\./%2e} ; i=${i//\//%2f}  ; i=${i//:/%3a}
        i=${i//;/%3b}  ; i=${i//</%3c}   ; i=${i//=/%3d}
        i=${i//>/%3e}  ; i=${i//\?/%3f}  ; i=${i//@/%40}
        i=${i//\[/%5b} ; i=${i//\\/%5c}  ; i=${i//\]/%5d}
        i=${i//\^/%5e} ; i=${i//_/%5f}   ; i=${i//\`/%60}
        i=${i//\{/%7b} ; i=${i//|/%7c}   ; i=${i//\}/%7d}
        i=${i//\~/%7e}
        echo "$i"
        i=""
}


# Read stdin and create the string of Grandstream config values


# Valid Key mode values:
# 0 = Speed Dial, 1 = BLF, 2 = Presence Watcher, 3 = eventlist BLF,
# 4 = Speed Dial via active account, 5 = DialDTMF, 6 = Voicemail, 
# 7 = CallReturn, 8 = Transfer, 9 = CallPark, 10 = Intercom, 
# 11 = LDAP Search

Pvalues=""
i=0
while read -t1 line
do
	name="${line%%'<'*}"
	extension="${line##*'<'}"
	extension="${extension%%'>'*}"
	if [ -z "$name" ]; then # Turn off any unused/old BLF 
		Pvalues+="P$((6001 + i))=0&" # Key mode, speed dial
	else
		Pvalues+="P$((6001 + i))=1&" # Key mode, BLF
	fi
	Pvalues+="P$((6201 + i))=0&" # Account
	Pvalues+="P$((6401 + i))=$(urlencode "$name")&" # Name
	Pvalues+="P$((6601 + i))=$(urlencode "$extension")&" # UserID
	((i++))
done

if [ -z "$Pvalues" ] || [ -z "$GXP_PHONE_IP" ]; then
	echo "Usage: stdout |$0 <IP Address of Grandstream GXP2020EXT device> [password]" >&2
	echo -e "\tSupply a list of names and extensions (one per line) via stdio in the format:" >&2
	echo -e "\tName <Extension>" >&2
	exit 1
fi

# Provision the GXP2020EXT

# Older firmware passed password as "P2", newer firmware passes as "password"
# <form> action is the same
sid="$(curl -s -i -d "P2=$GXP_PASSWD&password=$GXP_PASSWD" "http://${GXP_PHONE_IP}/cgi-bin/dologin" |get_sid)"
echo "Sid: $sid"

# <form> action for config update is different across firmware versions
# Try old firmware style first
status="$(curl -s -i -b "session_id=$sid" -d "${Pvalues}update=Update" "http://${GXP_PHONE_IP}/cgi-bin/update" |post_status)"
echo "Status: $status"

# Try new firmware way if the last attempt failed
if [ -z "$status" ]; then
#	# Try new firmware config update action
	status="$(curl -s -i -b "session_id=$sid" -d "$Pvalues&sid=$sid" "http://${GXP_PHONE_IP}/cgi-bin/api.values.post" |post_status)"
	echo "Status: $status"
fi
if [ -z "$status" ]; then
	# Couldn't update speed dial on phone
	echo "Could not update speed dial correctly"
	exit 1
else
	echo "Done."
fi
